const cmdLineOpt = require('@unitybase/base').options
const base = require('@unitybase/base')
const argv = base.argv
const assert = require('assert')

module.exports = function testAclSkipOnDelete (options) {
  if (!options) {
    const opts = cmdLineOpt.describe('', 'test SQL Injection')
      .add(argv.establishConnectionFromCmdLineAttributes._cmdLineParams)
    options = opts.parseVerbose({}, true)
    if (!options) return
  }
  const session = argv.establishConnectionFromCmdLineAttributes(options)
  const conn = session.connection
  // if overridden tst_document.select is called with __jsDisable
  const ACCESS_DENY = /<<<Access deny>>>/
  const NOT_VERIFIED_UBQL = 'UBQL must be verified before passing to method'

  assert.throws(
    () => conn.Repository('tst_document')
      .using('runSelectInJSContext')
      .attrs('ID', "(select ID from uba_user where name = 'admin')")
      .limit(1).selectAsObject(),
    ACCESS_DENY,
    'Must receive Access deny for unsafe fieldList'
  )
  assert.throws(
    () => conn.Repository('tst_document')
      .using('runSelectInJSContext')
      .attrs('[ID]//')
      .misc({__noClone: true})
      .limit(1).selectAsObject(),
    ACCESS_DENY,
    'Must receive Access deny for unsafe fieldList'
  )

  assert.throws(
    () => conn.Repository('uba_user')
      .attrs('*')
      .limit(1).selectAsObject(),
    ACCESS_DENY,
    'Must receive Access deny for unsafe * fieldList (restricted attributes)'
  )

  assert.throws(
    () => conn.Repository('tst_document')
      .using('runSelectInJSContext')
      .attrs('ID')
      .where("(select ID from uba_user where name = 'admin'", '=', 1)
      .limit(1).selectAsObject(),
    ACCESS_DENY,
    'Must receive error for unsafe where'
  )

  assert.throws(
    () => conn.Repository('tst_document')
      //.using('runSelectInJSContext')
      .attrs('ID')
      .where('[ID]//', '=', 1)
      .limit(1).selectAsObject(),
    ACCESS_DENY,
    'Must receive error for unsafe where'
  )

  assert.doesNotThrow(
    () => conn.Repository('tst_document')
      .using('runSelectInJSContext')
      .attrs('ID')
      .where('0=1', 'custom')
      .where('1=1', 'custom')
      .selectAsObject(),
    ACCESS_DENY,
    'Must allow where with 0=1 and 1=1'
  )

  assert.doesNotThrow(
    () => conn.Repository('tst_document')
      .using('runSelectInJSContext')
      .attrs('COUNT(1)')
      .where('person', 'isNull')
      .selectAsObject(),
    ACCESS_DENY,
    'Must allow COUNT(1) in fieldList'
  )

  assert.throws(
    () => conn.Repository('tst_document')
      .using('runSelectInJSContext')
      .attrs('ID')
      .orderBy("(select ID from uba_user where name = 'admin')")
      .limit(1).selectAsObject(),
    ACCESS_DENY,
    'Must receive Access deny for order item without attribute'
  )
  assert.throws(
    () => conn.Repository('tst_document')
      .using('runSelectInJSContext')
      .attrs('ID')
      .orderBy('UNSAFE_FUNC([code])')
      .limit(1).selectAsObject(),
    ACCESS_DENY,
    'Must receive Access deny for order item with attribute expression'
  )

  assert.throws(
    () => conn.query({
      entity: 'tst_document',
      method: 'runSelectInJSContext',
      fieldList: ['ID', 'code'],
      whereList: { c1: { expression: '[code]//', condition: 'equal', value: '1' } }
    }),
    ACCESS_DENY,
    'Must receive Access deny for unsafe ubql'
  )

  assert.throws(
    () => conn.Repository('uba_user').attrs('ID')
      .where('ID', '>', 0, 'A01')
      .where('ID', '>', 1, 'A02')
      .logic('([A01 OR [A02])')
      .select(),
    ACCESS_DENY,
    'Must receive Access deny for invalid logical predicate'
  )
  assert.doesNotThrow(
    () => conn.Repository('uba_user').attrs('ID')
      .where('ID', '>', 0, 'A01')
      .where('ID', '>', 1, 'A02')
      .logic('[A01] OR [A02]')
      .select(),
    ACCESS_DENY,
    'Should NOT receive Access deny for not () logical predicate'
  )
  assert.throws(
    () => conn.Repository('uba_user').attrs('ID')
      .where('ID', '>', 0, 'A01')
      .where('ID', '>', 1, 'A02')
      .logic('([A01]) OR [A02])')
      .select(),
    ACCESS_DENY,
    'Must receive Access deny for not matched () in logical predicate'
  )
  assert.throws(
    () => conn.Repository('uba_user').attrs('ID')
      .where('ID', '>', 0, 'A01')
      .where('ID', '>', 1, 'A02')
      .logic('(([A01]) XOR [A02]))')
      .select(),
    ACCESS_DENY,
    'Must receive Access deny for not AND/OR in logical predicate'
  )

  assert.throws(
    () => conn.Repository('uba_user').attrs('ID')
      .orderBy('[name]//')
      .select(),
    ACCESS_DENY,
    'Must receive Access deny for not AND/OR in logical predicate'
  )
}
