const path = require('path')
const VueLoaderPlugin = require('vue-loader/lib/plugin')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')

module.exports = {
  resolve: {
    extensions: ['.js', '.vue', '.json'],
    alias: {
      vue: 'vue/dist/vue.common.js'
    },
    // prevent using `exports` from package.json (if used - throttle-debounce cause error because element uses v1 but ub v5)
    exportsFields: [''],
    // 2 line below force webpack to lookup modules in docs_adminui/node_modules first
    // instead of resolve a same module (@unitybase/ub-pub for example) from several
    // symlinked locations created by `lerna link`
    symlinks: false,
    modules: [path.resolve(__dirname, '..', 'node_modules'), 'node_modules']
  },
  module: {
    rules: [{
      test: /\.vue$/,
      loader: 'vue-loader'
    },
    {
      test: /\.js$/,
      use: ['babel-loader'],
      exclude: /node_modules/
    },
    {
      test: /\.css$/,
      use: [
        MiniCssExtractPlugin.loader,
        'css-loader'
      ]
    },
    {
      test: /\.(woff(2)?|ttf|eot|svg)(\?v=\d+\.\d+\.\d+)?$/,
      use: [{
        loader: 'file-loader',
        options: {
          name: '[name].[ext]',
          outputPath: 'fonts/'
        }
      }]
    }]
  },

  plugins: [
    new VueLoaderPlugin(),
    new MiniCssExtractPlugin({
      filename: 'style.css'
    })
  ],
  watchOptions: {
    followSymlinks: true,
    //poll: 10000, // Check for changes every 10 seconds
  }
}
