`$App.doCommand({cmdType: 'showList', ....})` shows grid with entity rows

Json below can be passed programmatically to `$App.doCommand` or saved as navigation shortcut command text
(click on navigation shortcut actually call `$App.doCommand(commandText)`)

The simplest config will show all entity attributes without `defaultView: false` in meta:
```json
{
  "cmdType": "showList",
  "cmdData": {
    "entityName": "ubm_enum"
  }
}
```

More complex example with repository (wheres, orders, etc.) and custom columns definition 
```json
{
  "renderer": "vue",
  "cmdType": "showList",
  "cmdData": {
    "repository": {
      "entity": "ubm_enum",
      "fieldList": ["eGroup", "sortOrder", "code", "shortName", "name"],
      "orderList": {
        "eGroup": { "expression": "eGroup", "order": "asc" },
        "sortOrder": { "expression": "sortOrder", "order": "asc" }
      }
    },
    "columns": ["eGroup", {
      "id": "code",
      "width": 120,
      "format": "return value * 2"
    }]
  }
}
```

### Parameters
 - renderer: string - renderer type (optional). One of  'ext' or 'vue'; Default value depends on `ubConfig.uiSettings.adminUI.useVueTables` parameter
   - cmdData: Object - showList command parameters
     - entityName: string - if `entityName` is passed, then `columns` and `fieldList` will be computed automatically. Ignored if `repository` is defined
     - repository: Object - `ubql` for getting rows
     - columns: array<string|object|> - optional columns definition. If not passed - will be generated using fieldList attributes
       Column can be either a string with attribute name, or object of type [UTableColumn](/api/ubpub-v5/module-Types.html#UTableColumn)
       See also `UTableEntity.columnTemplates` for global column templates.
      
       It is possible to define entityName and columns only - in this case only defined columns will be displayed
        ```json
        {
          "cmdType": "showList",
          "cmdData": {
            "entityName": "ubm_enum",
            "columns": ["eGroup", "code"]
          }
        }
        ```
     - scopedSlots: function - Provide custom slots to showList
```javascript
{
  "cmdType": "showList",
  "cmdData": {
    "entityName": "ubm_enum",
    "columns": ["eGroup", "code"],
    scopedSlots: createElement => ({
      toolbarAppend: (slotScope) => createElement('button', 'click me'),
      eGroup: ({ row }) => createElement(
        'u-button',
        {
          props: {
            icon: 'u-icon-send'
          }
        },
        row.ID + row.eGroup
      )
    })
  }  
}
```

### Other UTableEntity options
Almost all `UTableEntity` options can be passed using cmdData. Example below hide "Filter button" and define `buildEditConfig` handler  
```javascript
{
  cmdType: "showList",
  cmdData: {
    entityName: "ubm_enum",
    columns: ["eGroup", "code"],
    hideActions: ['filter'],
    buildEditConfig (cfg, row) {
      cfg.entity = row.entity
      return cfg
    }
  }  
}
```
