`$App.showModal({cmdType: 'showForm', formCode: '...',...})` creates and shows form
(one of `ubm_form`) in modal mode and returns promise, what will be resolved by form

Example:

```javascript
 const choice = await UB.core.UBApp.showModal({
    cmdType: 'showForm',
    formCode: 'some-form-code',
    isModal: true,
    modalWidth: '700px'
  })
  if (!choice.success) {
    throw new UB.UBAbortError('canceled by user')
  }
```

And form definition

```javascript
<template>
<div
  v-loading="loading"
  class="u-form-layout"
>
  <u-form-container
    label-position="top"
  >
    <u-form-row label="Номер">
      <u-base-input v-model="partNumber" />
    </u-form-row>
    <u-button @click="resolveParams">
      Process
    </u-button>
  </u-form-container>
</div>
</template>

<script>
const { Form } = require('@unitybase/adminui-vue')
const UB = require('@unitybase/ub-pub')

module.exports.mount = function (cfg) {
  Form({
    ...cfg,
    props: {
      deferred: cfg.deferred // pass deffered: {resolve: function, reject: function} into form props
    }
  }).mount()
}

module.exports.default = {
  name: 'some-form-code',
  inject: ['$formServices'],
  props: {
    deferred: Object
  },
  data () {
    return {
      loading: false,
      isResolved: false,
      partNumber: ''
    }
  },
  methods: {
    resolveParams () {
      this.isResolved = true
      this.$props.deferred.resolve({
        success: true,
        params: {
          partNumber: this.partNumber
        }
      })
      this.$formServices.forceClose()
    }
  },
  /** if user close form w\o pressing `Process` button - resolve result to {success: false} */
  beforeDestroy () {
    if (!this.isResolved) {
      this.$props.deferred.resolve({
        success: false
      })
    }
  }
}
</script>
```
