### Basic usage
```vue
<template>
<u-file-multiple
  v-model="files"
  file-attribute="photo"
  subject-attribute="request"
  entity-name="req_attachment"
  :subject-attribute-value="1"
/>
</template>
<script>
export default {
  data () {
    return {
      files: []
    }
  }
}
</script>
```

### Disabled
```vue
<template>
<u-file-multiple
  v-model="files"
  file-attribute="photo"
  subject-attribute="request"
  entity-name="req_attachment"
  :subject-attribute-value="1"
  disabled
/>
</template>
<script>
export default {
  data () {
    return {
      files: []
    }
  }
}
</script>
```

### View mode
```vue
<template>
<div>
  <u-radio v-model="viewMode" :items="viewModes" align="horizontal"/>
  
  <u-file-multiple
    v-model="files"
    file-attribute="photo"
    subject-attribute="request"
    entity-name="req_attachment"
    :subject-attribute-value="1"
    :view-mode="viewMode"
  />
</div>
</template>
<script>
export default {
  data () {
    return {
      files: [],
      viewMode: 'table',
      viewModes : [
        { id: 'table', label: 'Table' },
        { id: 'carousel', label: 'Carousel' },
        { id: 'carouselWithPreview', label: 'Carousel with preview' }
      ]
    }
  }
}
</script>
```

### Custom additional button
```vue
<template>
<u-file-multiple
  v-model="files"
  file-attribute="photo"
  subject-attribute="request"
  entity-name="req_attachment"
  :subject-attribute-value="1"
>
  <u-button
    appearance="inverse"
    icon="u-icon-send"
  >
    Test
  </u-button>
</u-file-multiple>
</template>
<script>
export default {
  data () {
    return {
      files: []
    }
  }
}
</script>
```

### Access to parent UFile instance from custom button
```vue
<template>
<u-file-multiple
  v-model="files"
  file-attribute="photo"
  subject-attribute="request"
  entity-name="req_attachment"
  :subject-attribute-value="1"
>
  <custom-button/>
</u-file-multiple>
</template>
<script>
export default {
  data () {
    return {
      files: []
    }
  }
}
</script>
```

### Custom button component
```vue
<template>
<u-button
    appearance="inverse"
    icon="u-icon-send"
    :disabled="instance.file || instance.disabled"
    @click="showParentInstance"
>
  Test
</u-button>
</template>
<script>
export default {
  inject: {
    instance: 'fileComponentInstance'
  },

  methods: {
    showParentInstance () {
      console.log(this.instance)
    }
  }
}
</script>
```

### Remove default buttons

```vue
<template>
<u-file-multiple
  v-model="files"
  file-attribute="photo"
  subject-attribute="request"
  entity-name="req_attachment"
  :subject-attribute-value="1"
  remove-default-buttons
/>
</template>
<script>
export default {
  data () {
    return {
      files: []
    }
  }
}
</script>
```

To remove one or few buttons pass array with button names:
- add
- webcam
- scan
- scanSettings
- download
- remove
- preview
- fullscreen

```vue
<template>
<u-file-multiple
  v-model="files"
  file-attribute="photo"
  subject-attribute="request"
  entity-name="req_attachment"
  :subject-attribute-value="1"
  :remove-default-buttons="['add', 'preview']"
/>
</template>
<script>
export default {
  data () {
    return {
      files: []
    }
  }
}
</script>
```

### Addition Slot

```vue
<template>
  <u-file-multiple
    v-model="files"
    file-attribute="photo"
    subject-attribute="request"
    entity-name="req_attachment"
    :subject-attribute-value="1"
    remove-default-buttons
  >
    <template #additionSidebar="{selectedFileId}">
      <div>
        Active dictID is: {{ files.find(i => i.ID === selectedFileId).dictID}}
      </div>
    </template>
  </u-file-multiple>
</template>

<script>
export default {
  data () {
    return {
      files: []
    }
  }
}
</script>
```
