# Опис
Фабрична ф-я `Form` створює стор для форми і рендерить її в таб або модальне вікно.
Стор відстежує зміни форми і будує запити на зміну форми.
У формі доступна валідація та її перевірка перед збереженням (див. `Validation tutorial`).

> See real form code - [tst_dictionary-ft-fm.vue](https://git-pub.intecracy.com/unitybase/ubjs/-/blob/master/apps/autotest/models/TST/public/forms/tst_dictionary-ft-fm.vue)

## Приклад мінімального використання

```javascript
const { Form } = require('@unitybase/adminui-vue')

module.exports.mount = function ({ title, entity, instanceID, rootComponent }) {
  Form({
    component: rootComponent,
    entity,
    instanceID,
    title
  }).mount()
}
module.exports.default ={
  name: 'MyCustomVueComponent'
}
```

## Приклад повного використання
```javascript
module.exports.mount = function ({ title, entity, instanceID, rootComponent }) {
  Form({
    component: rootComponent,
    entity,
    instanceID,
    title: 'Any custom title',
    isModal: true,
    modalClass: 'test-class', // кастомний клас для модального вікна
    modalWidth: '900px', // ширина модальногі вікна
    formCode
  })
    .store({ // .store можна не використовувати, якщо не потрібно мати додаткові параметри
      state: {},
      getters: {},
      mutations: {},
      //...
    })
    .processing({
      // приклад хука, список хуків описано нижче в processing
      async beforeSave (store) {
        return $App.dialogYesNo('custom before save', 'are you sure?')
      },
      masterFieldList: ['ID', 'name', 'age', 'phone'], // можно не указывать
      /**
       * collections - запити для пов'язаних із формою записів.
       * Приймає дані виду {[ключ колекції]: [UB repository]}
       * 
       * Якщо потрібно, щоб дані в колекції були завантажені не відразу, а за якоюсь подією, то можна додати lazy колекцію
       * вказавши дані в такому вигляді.
       * 
       * При необхідності виконувати оновлення колекції ДО майстер запису - використовуєте ключ `beforeMaster: true`  
       * 
       * {
       *    [ключ колекції]: {
       *      repository: [UB repository],
       *      lazy: true,
       *      //beforeMaster: true
       *    }
       * }
       * У цьому випадку дані будуть завантажені тільки після виклику екшену loadCollections,
       * який приймає в себе масив ключів колекцій, приклад: loadCollections(['todo'])
       *
       * Для колекції "participants" вказані функції, які будують запит на створення записів
       * і обробка відповідей від цих запитів, а також видалення записів, замість логіки за замовчуванням.
       */
      collections: {
        todo: {
          repository: ({state}) => UB.connection
            .Repository('tst_dictionary_todo')
            .attrs('ID', 'objectID', 'name', 'status', 'link')
            .where('objectID', '=', state.data.ID),
          lazy: true
        },
          
        dueDate: () => UB.connection
          .Repository('tst_due_date')
          .attrs('ID', 'dateFrom', 'dateTo', 'status')
          .where('dateTo', '<', Date.now()),

        participants: {
          repository: ({state}) => Repository('dfx_Document_ppt')
            .attrs(
              'ID',
              'objectID',
              'subjectID',
              'role'
            )
            .where('objectID', '=', state.data.ID)
            .where('role', 'notIn', inAttrRoles),
          buildRequest({state, collection, fieldList, item}) {
            return {
              entity: 'dfx_Document',
              method: 'addParticipant',
              fieldList,
              execParams: {
                ID: state.data.ID,
                subjectID: item.data.subjectID,
                role: item.data.role
              },
              collection: collection.key
            }
          },
          handleResponse ({ commit, collection, response }) {
            const loadedState = response.resultData
            for (const loadedItem of loadedState) {
              const index = collection.items.findIndex(i => i.data.subjectID === loadedItem.subjectID)
              if (index !== -1) {
                commit('LOAD_COLLECTION_PARTIAL', {
                  collection: 'participants',
                  index,
                  loadedState: loadedItem
                })
              }
            }
          },
          buildDeleteRequest({state, item}) {
            return {
              entity: 'dfx_Document',
              method: 'removeParticipant',
              execParams: {
                ID: state.data.ID,
                subjectID: item.data.subjectID
              }
            }
          }
        }
      }
    })
    .validation()
    .mount()
}

module.exports.default ={
  name: 'MyCustomVueComponent'
}
```

## Прокидання значень за замовчуванням у форму для нового запису
Forms приймає на вхід опціональний параметр `props.parentContext` - object зі значеннями атрибутів за замовчуванням для
методу `addNew`.

Таким чином можна, наприклад, передати дочірній формі значення ідентифікатора запису поточної форми (майстер-деталь).
`$App.doCommand` прокидає props транзитом у конструктор Form.

У прикладі нижче ми викликаємо форму в режимі додавання запису (не передаємо instanceID), при цьому значення атрибута `docID`
нової форми буде 123 (якщо його не змінить серверна реалізація методу `addNew`)

```javascript
  $App.doCommand({
    cmdType: 'showForm',
    entity: 'doc_controltask',
    formCode: 'doc_controltask_form',
    isModal: true,
    props: {
      parentContext: { docID: 123 }
    }
  })
```  

## Попередження
> `instance -> processing -> validation` мають використовуватися послідовно.
> Тобто можна використовувати instance окремо, але processing без instance не можна, те саме стосується і validation

### Instance
Хранит данные формы и отслеживает изменения в ней.
Все взаимодействие со стором рекомендуется делать используя мутации описаные ниже.

#### state
 - `data` - it is an object with actual (to be shown on UI) data values, regardless if values are untouched by user 
 or already edited.
 - `originalData` - his object is initially empty, but as user starts editing, it is filled by original values, as they 
 loaded from DB, so that it would be always possible to say if a certain attribute was changed or not. 
 If after some editing, value returned to its original state, value is deleted from this object. 
 When this object is has no attributes, we know there is nothing to save.
 - `collections` - this is a property for complex object, objects which consist of one master record and collection
 or multiple collections of detail records. Each collection tracks added, changed and deleted items, so that we know
 if there is any change to save in the collection. Collection item is tracked just like the master record, using the
 same technique - "data" and "originalData" properties for item.  Item also has "isNew" property, indicating if item
 was added after original loading of collection or not. The "deleted"

#### getters
 - `isDirty` геттер, вернет `true` если в форме были изменены какие либо данные

#### mutations
 - `SET_DATA`: вносит изменения в `data`
 - `ASSIGN_DATA`: тоже что и `SET_DATA` только для нескольких значений сразу
 - `SET`: вносит изменения в остальные опции стора (не `data`), не влияет на отслеживание изменений
 - `LOAD_DATA`: принимает в себя объект с данными формы и записывает в `data`, мутация очищает `originalData`
 поэтому форма будет `isDirty === false`
 - `LOAD_DATA_PARTIAL`: догружает данные в `data`, чистит из `originalData` только те поля которые были переданы
 - `LOAD_COLLECTION`: тоже что и `LOAD_DATA` только для коллекции
 - `LOAD_COLLECTION_PARTIAL`: тоже что и `LOAD_DATA_PARTIAL` только для коллекции
 - `ADD_COLLECTION_ITEM`: добавляет новый элемент в коллецию. Вместо этой мутации рекомендуюется использовать
 `action -> addCollection` из модуля **processing**  
 - `DELETE_COLLECTION_ITEM`: убирает элемент из коллекции и добавляет его в массив `deleted`,
 после сохранения формы для всех элементом массива `deleted` будут отправлены запросы на удаление. 
 - `DELETE_COLLECTION_ITEM_WITHOUT_TRACKING`: убирает элемент из коллекции и НЕ добавляет его в массив `deleted`,
 запросы на удаление НЕ будут отправлены на сервер при сохранении.  Может быть полезно в случае, когда известно, что
 сервер при сохранении сам удаляет записи, например, при каскадном удалении делаталей.  
 - `CLEAR_ALL_DELETED_ITEMS`: очищает массив `deleted` у всех коллекций, чаще всего вам не прийдется его использовать,
 так как он вызывается автоматически после сохранения формы в модуле **processing**
 - `DELETE_ALL_COLLECTION_ITEMS`: добавляет все элементы коллекции в массив удаления, после сохранения формы для всех
 элементом массива `deleted` будут отправлены запросы на удаление. 

### Processing
Створює в сторі CRUD екшени, в яких лежать UB запити, побудовані на основі instance-module data

#### state
- `isNew`: вказує чи нова це форма
- `formCrashed`: вказує чи була помилка при завантаженні форми

#### getters
- `loading`: true якщо відбувається будь-яке завантаження у формі
- `canRefresh`: true якщо у користувача є доступ до оновлення форми
- `canSave`: true якщо у користувача є доступ до збереження форми
- `canDelete`: true якщо у користувача є доступ до видалення форми

#### mutations
- `LOADING` - додає/виключає прапор із масиву завантаження.
 > Пример 
 ```javascript
  commit('LOADING', {
    isLoading: true,
    target: 'loadSomeRepo'
  })

  await UB.Repository('some_repo')
    .attrs('*')
    .select()

  commit('LOADING', {
    isLoading: false,
    target: 'loadSomeRepo'
  })
  ```

#### actions
- `deleteInstance`: виводить yes/no діалогове вікно, при кліці на yes видаляє запис
- `refresh`: оновлює дані у формі
- ``save`: зберігає форму
- `loadCollections`: дані в таких колекціях будуть завантажені тільки тоді, коли вручну буде викликаний цей екшн.
  Це зроблено спеціально, щоб завантажувати пов'язані записи тільки тоді, коли вони необхідні,
  наприклад, коли дані запису колекції знаходяться на вкладці, яка не ввімкнена під час відкриття форми.

#### hooks
Хук може бути звичайною функцією або async (функція, яка повертає проміс)

 - **beforeInit**
 - **inited**
 - **beforeSave**: якщо функція поверне false save не буде виконано
 - **saved**
 - **beforeCreate** 
 - **created**
 - **beforeLoad** 
 - **loaded**
 - **beforeDelete**
 - **deleted**

### Validation
Дивіться [Validation tutorial](Validation%20tutorial).

На основі даних в instance модулі та entitySchema створює об'єкт валідації `$v` - [vuelidate](https://github.com/vuelidate/vuelidate)
отриманий об'єкт через provide прокидається в компонент.
Можна отримати в будь-якому дочірньому компоненті форми за допомогою `inject: ['$v']`.
Можна передати Vue mixin як параметр, в якому можна буде перетереть стандартну валідацію.
  
#### Пример кастомной валидации
  dynamicField буде обов'язковим, якщо someNumber > 25
    
```javascript
const { required, between } = require('vuelidate/lib/validators/index')

module.exports.mount = function ({ title, entity, instanceID, formCode, rootComponent }) {
  Form({
    component: rootComponent,
    entity,
    instanceID,
    title,
    formCode
  })
  .processing()
  .validation({
    computed: {
      name () {
        return this.$store.state.data.name
      },
      
      someNumber () {
        return this.$store.state.data.someNumber
      },
      
      dynamicField () {
        return this.$store.state.data.dynamicField
      },
      
      'dynamicField:caption'() {
        return this.$ut('some.i18n.key.for.dynamicField')
      } 
    },
  
    validations () {
      return {
        name: { required },
        someNumber: {
          required, 
          between: between(20, 30)
        },
        ...(this.someNumber > 25 ? {
          dynamicField: { required }
        } : {})
      }
    }
  })
  .mount()
}

```


## Form template recommendations

```html
<template>
  <!-- 
    u-form-layout - a class that stretches the form to the height of the window and does the right job of scrolling. 
    
    The `v-loading` directive is recommended to be used at the top level so that the toolbar overlaps.
    
    The `v-loading.body` modifier is not recommended because it will overlap the given area on all opened forms
  -->
  <div
    v-loading="loading"
    class="u-form-layout">
    <!-- тулбар-->
    <u-toolbar />

    <!-- u-form-container provides label-width for all child u-form-row  -->
    <u-form-container :label-width="160">
      <!-- creates a u-form-row with child control based on entity attribute type -->
      <u-auto-field attribute-name="code" />
      
      <!-- the same as above, but manually defined  -->
      <u-form-row
        :label="getLabel('code')"
        required
        :error="$v.code.$error"
      >
        <u-base-input v-model="code" />
      </u-form-row>

      
    </u-form-container>
  </div>
</template>

<script>
const { mapInstanceFields } = require('@unitybase/adminui-vue')
const { mapGetters } = require('vuex')

export default {
  inject: ['$v'], // validation, 

  computed: {
    ...mapInstanceFields(['code', 'caption']), // helper to get/set a form data directly, without digging into store
    ...mapGetters(['loading'])
  }
}
</script>
```
