const csvLoader = require('@unitybase/base').dataLoader
const path = require('path')
/**
 * Fill CDN dictionaries from csv files
 * Used by `ubcli initialize` command
 * @param {ServerSession} session
 */
module.exports = function (session) {
  let conn = session.connection

  console.info('\tFill common dictionaries data (CDN model)')

  console.info('\t\tCity types (cdn_citytype)')
  csvLoader.loadSimpleCSVData(conn, path.join(__dirname, 'cdn_citytype.csv'), 'cdn_citytype', 'code;name'.split(';'), [0, 1], 1)

  console.info('\t\tRegion types (cdn_regiontype)')
  csvLoader.loadSimpleCSVData(conn, path.join(__dirname, 'cdn_regiontype.csv'), 'cdn_regiontype', 'code;name'.split(';'), [0, 1], 1)

  console.info('\t\tCountries (cdn_country)')
  csvLoader.loadSimpleCSVData(conn, path.join(__dirname, 'cdn_country.csv'), 'cdn_country', 'code;name;fullName;intCode;symbol2;symbol3;phoneCode'.split(';'), [4, 0, 1, 2, 3, 4, 5], 1)

  console.info('\t\tRegions of Ukraine (cdn_region)')
  var ukraineID = conn.lookup('cdn_country', 'ID', {expression: 'code', condition: 'equal', value: 'UKR'})
  if (!ukraineID) {
    throw new Error('Country with code UKR not found')
  }

  // CSV columns: code,regionType,name,fullName
  csvLoader.loadSimpleCSVData(conn, path.join(__dirname, 'cdn_region_ukraine.csv'), 'cdn_region',
    ['parentAdminUnitID', 'code', 'regionTypeID', 'name', 'caption', 'fullName'],
    [
      function () { return ukraineID },
      0,
      function (row) {
        var regionType
        regionType = conn.lookup('cdn_regiontype', 'ID', {expression: 'code', condition: 'equal', values: {code: row[1]}})
        if (!regionType) {
          throw new Error('Unknown region type ' + row[1])
        }
        return regionType
      }, 2, 2, 3
    ], 1, ','
  )

  console.info('\t\tContact types (cdn_contacttype)')
  csvLoader.loadSimpleCSVData(conn, path.join(__dirname, 'cdn_contacttype.csv'), 'cdn_contacttype', 'code;name'.split(';'), [0, 1], 1)

  console.info('\t\tCurrency (cdn_currency)')
  csvLoader.loadSimpleCSVData(conn, path.join(__dirname, 'cdn_currency.csv'), 'cdn_currency', 'name;intCode;code3;curMult'.split(';'), [0, 1, 2, 3], 1)

  console.info('\t\tLanguage list (cdn_language)')
  csvLoader.loadSimpleCSVData(conn, path.join(__dirname, 'cdn_language.csv'), 'cdn_language',
    'code;languageName;nativeName;isRTL'.split(';'),
    [
      0, 1, 2, 3
    ],
    1)
}
