module.exports = generateUbqlForCdnClassifier

/**
 * @param {object} params
 * @param {SyncConnection} params.conn
 */
function generateUbqlForCdnClassifier ({ conn }) {
  console.log('Migration script start:', __filename)

  const entity = 'cdn_classifier'

  console.info(
    `  Update all "${entity}.ubql" attribute from autogenerated ubql using "UB.Repository"`
  )

  console.log(`\tLoading entries of ${entity}...`)
  const entries = conn.Repository(entity).attrs('ID', 'code', 'orderByAttr').select()
  console.log(`\t${entries.length} entries`)

  entries.forEach(({ ID, code, orderByAttr }) => {
    const generatedUbql = JSON.stringify(
      conn.Repository('cdn_classifieritem')
        .attrs('ID', 'code', 'name')
        .where('classifierID', '=', ID)
        .orderBy(orderByAttr)
        .ubql()
    )

    conn.query({
      entity,
      method: 'update',
      execParams: {
        ID,
        ubql: generatedUbql
      },
      __skipOptimisticLock: true
    })
    console.info('\tSet ubql="%s" for "%s"', generatedUbql, code)
  })

  console.log('Migration script finish:', __filename)
}
