module.exports = migrate

const fs = require('fs')
const path = require('path')
const { csv, dataLoader } = require('@unitybase/base')

/**
 * @param {object} params
 * @param {SyncConnection} params.conn
 */
function migrate ({ conn }) {
  console.log('Migration script start:', __filename)

  // read csv
  const filePath = path.join(__dirname, '..', '..', '_initialData', 'cdn_country.csv')
  const csvStr = fs.readFileSync(filePath, { encoding: 'utf-8' })
  const rows = csv.parse(csvStr).slice(1)
  const data = rows.reduce((res, row) => {
    res[row[4]] = row[5] // { symbol3: phoneCode }
    return res
  }, {})

  // update data
  const entity = 'cdn_country'
  const countries = conn.Repository(entity).attrs('code').selectAsArrayOfValues()

  for (const countryCode in data) {
    // update phone codes
    const phoneCode = data[countryCode]
    if (countries.includes(countryCode) && phoneCode) {
      conn.xhr({
        endpoint: 'runSQL',
        URLParams: { connection: 'main' },
        data: `UPDATE ${entity} SET phoneCode = '${phoneCode}' WHERE code = '${countryCode}';`
      })
    }
  }

  // add new data. In case insertion fails because dictionary is edited by hands - skip adding
  const newRows = rows.filter(row => !countries.includes(row[4]))
  try {
    dataLoader.loadArrayData(conn, newRows, entity, 'code;name;fullName;intCode;symbol2;symbol3;phoneCode'.split(';'), [4, 0, 1, 2, 3, 4, 5])
  } catch (e) {
    console.error('Fail to add a new countries, may be cdn_country dictionary diffs from cdn_country.csv')
  }
  console.log('Migration script finish:', __filename)
}
