const UB = require('@unitybase/ub')
const http = require('http')
/* global cdn_bank cdn_city */
// eslint-disable-next-line camelcase
const me = cdn_bank

me.entity.addMethod('syncWithNBU')
me.on('insert:before', setDescriptionAttribute)
me.on('update:before', setDescriptionAttribute)
cdn_city.on('update:after', setDescriptionAttributeByCity)

function setDescriptionAttribute (ctx) {
  const instanceData = ctx.dataStore.getAsJsObject()[0] || {}
  const execParams = ctx.mParams.execParams
  const cityName = execParams.cityID || instanceData.cityID
    ? (UB.Repository('cdn_city').attrs('name')
      .misc({ __allowSelectSafeDeleted: true })
      .selectById(execParams.cityID || instanceData.cityID)).name || ''
    : ''

  execParams.description =
    (execParams.MFO || instanceData.MFO || '') + ' ' +
    (execParams.name || instanceData.name || '') + ' ' +
    cityName
}

function setDescriptionAttributeByCity (ctx) {
  const bankStore = UB.DataStore('cdn_bank')
  const cdnAccount = UB.Repository('cdn_bank')
    .attrs(['ID', 'mi_modifyDate'])
    .where('cityID', 'equal', ctx.mParams.execParams.ID)
    .misc({ __allowSelectSafeDeleted: true })
    .selectAsObject()
  cdnAccount.forEach((item) => {
    // trigger update:before to set a description
    bankStore.run('update', {
      execParams: {
        ID: item.ID,
        mi_modifyDate: item.mi_modifyDate,
        description: ''
      }
    })
  })
}

/**
 * Add's missed Ukrainian banks from dictionary, provided by National Bank of Ukraine
 * @method syncWithNBU
 * @param {ubMethodParams} [ctx]
 * @param {Object} [ctx.mParams]
 * @memberOf cdn_bank_ns.prototype
 * @published
 */
me.syncWithNBU = function syncWithNBU (ctx) {
  let loaded = false
  let tries = 0
  let NB
  let errMsg
  while (!loaded && (tries < 5)) {
    try {
      NB = http.get('https://bank.gov.ua/NBU_BankInfo/get_data_branch',
        {
          typ: 0,
          json: ''
        }).json()
      loaded = true
    } catch (e) {
      tries++
      errMsg = e.message
    }
  }
  if (!loaded) {
    throw new Error(`5 невдачних спроб завантаження ${errMsg}`)
  }
  const ourB = UB.Repository('cdn_bank')
    .attrs('MFO').selectAsArrayOfValues()
  const ourBS = new Set(ourB)
  const cdnBank = UB.DataStore('cdn_bank')
  NB.forEach(b => {
    if (ourBS.has(b.MFO)) return
    cdnBank.run('insert', {
      __skipSelectAfterInsert: true,
      execParams: {
        MFO: b.MFO,
        name: b.SHORTNAME,
        fullName: b.FULLNAME,
        phones: b.TELEFON,
        address: `${b.TNP}${b.NP}} ${b.ADRESS}`
      }
    })
  })
}
