/**
 * Remove the access rights of the system role DataManager to the org. structure
 *
 * @param {SyncConnection} conn
 */
function removeDataManagerAccess ({ conn }) {
  console.log('Migration script start:', __filename)

  const dataManagerID = conn.Repository('uba_role')
    .attrs('ID')
    .where('name', '=', 'DataManager')
    .selectScalar()

  if (dataManagerID) {
    deleteNavRecords(conn, 'ubm_desktop', dataManagerID)
    deleteNavRecords(conn, 'ubm_navshortcut', dataManagerID)
    deleteRecords(conn, 'uba_els', conn.Repository('uba_els')
      .attrs('ID')
      .where('code', '=', 'ORG_FULL_DATAMANAGER')
      .selectAsArrayOfValues()
    )
  }
  console.log('Migration script finish: ', __filename)
}

/**
 * Delete records with code startsWith `org_` from `${entity}_adm`
 *
 * @param {SyncConnection} conn
 * @param {string} entity
 * @param {string} dataManagerID
 */
function deleteNavRecords (conn, entity, dataManagerID) {
  const records = conn.Repository(entity)
    .attrs('ID')
    .where('code', 'startWith', 'org_')
    .selectAsArrayOfValues()

  for (const ID of records) {
    const ids = conn.Repository(`${entity}_adm`)
      .attrs('ID')
      .where('instanceID', '=', ID)
      .where('admSubjID', '=', dataManagerID)
      .selectAsArrayOfValues()

    deleteRecords(conn, `${entity}_adm`, ids)
  }
}

/**
 * Delete multiple ids from entity
 *
 * @param {SyncConnection} conn
 * @param {string} entity
 * @param {Array} ids
 */
function deleteRecords (conn, entity, ids) {
  for (const ID of ids) {
    console.log('delete from %s record with ID = %d', entity, ID)
    conn.query({
      entity,
      method: 'delete',
      execParams: {
        ID
      }
    })
  }
}

module.exports = removeDataManagerAccess
