# systemjs-plugin-vue-ub

> Simple SystemJS plugin for Vue single file components. See Limitations below

Cloned from [systemjs-plugin-vue](https://github.com/vuejs/systemjs-plugin-vue) and modified by UnityBase team to be usable in browser.

Limitations:
 - scoped CSS not supported. We strongly recommend to use [BEM](http://getbem.com/introduction/) as css naming convention.
 For example if componentName is MyComponnet then Block name should be `my-component`:

```
<style>
  .my-component {
    font-size: 3em;
    text-align: center;
  }
</style>
```

- CSS preprocessors not supported - use BEM + CSS variables
- ES6 import not supported. Use CommonJS syntax. ES6 `export default` is supported for Vue SFC - parser will replace it to `module.exports.default =`)
- parser for script section is pretty dumb. To work correctly `module.exports.defauilf = {` or `export default {` section should exists EXACTLY (including spaces)
- vue component definition accessible inside `default` so should be imported as 
  `const MyComponent = require('./MyComponent.vue').default`


Working sample

```
// file /components/TstHello.vue

<template>
  <p class="hello">Hello, {{ whom }}!</p>
</template>

<script>
module.exports.default = {
  name: 'TstHello',
  props: {
    whom: {
      type: String,
      default: 'Unknown'
    }
  },
  data: function () {
    return { }
  }
}
</script>

<style>
.hello {
  font-size: 3em;
  text-align: center;
}
</style>

```

# Original readme

## !!! Maintenance Status !!!

This project is currently on hold, and it may not work properly with the latest Vue 2.0 versions. This is because the interest in using SystemJS with Vue doesn't seem to justify the effort required to maintain this plugin. If you are interested in taking over the maintenance of this project, please open an issue!

## Usage

This plugin is only tested with jspm 0.17+ and Vue.js 2.0+.

``` bash
jspm install npm:vue@2.0.0-alpha.5
jspm install --dev npm:systemjs-plugin-vue
```

``` js
System.config({
  "meta": {
    "*.vue": {
      "loader": "systemjs-plugin-vue"
    }
  }
})
```

You can use [this vue-cli template](https://github.com/vuejs-templates/systemjs) to scaffold an example project.

## Features

- ES2015 by default (requires `systemsjs-plugin-babel`)

- `lang="xxx"` pre-processors

- Scoped CSS

- PostCSS support

- CSS are automatically extracted across all components and injected as a single `<style>` tag

## Pre-Processors

To enable a pre-processor, you need to install the corresponding pre-processor module **via npm, not jspm**. Note that if you are using any non-babel pre-processors, then in-browser JIT compilation will not work due to Node dependencies. Use `jspm bundle -wid` during development, it's faster anyway.

Example:

``` bash
npm install less --save-dev
```
``` html
<style lang="less">
  /* use less! */
</style>
```

These are the preprocessors supported out of the box:

- stylus
- less
- scss (via `node-sass`, use `sass` in [config section](#configuring-options))
- jade
- pug
- coffee-script (use `coffee` in [config section](#configuring-options))

## Configuring Options

You can add a Vue section in your SystemJS config:

``` js
System.config({
  vue: {
    // options
  }
})
```

Or, alternatively, create a `vue.config.js` file at the root of your project, and export the configuration object.

### Passing Options to Pre-Processors

Just add a nested object under `vue`:

``` js
System.config({
  vue: {
    less: {
      paths: [...] // custom less @import paths
    }
  }
})
```

### Custom Lang Compiler

You can provide custom `lang` compilers under the `compilers` option. It is recommended to use `vue.config.js` for custom compilers, and in most cases you will want to import your Node dependencies as raw Node modules:

``` js
// vue.config.js
export default {
  compilers: {
    'my-lang' (raw, filename) {
      return System.import('@node/my-lang').then(myLang => {
        return myLang.compile(raw) // assumes returning a promise
      })
    }
  }
}
```

### PostCSS Configuration

Use `vue.postcss` in your SystemJS config file. The option can be:

- An array of plugins. Each plugin can either be a string module name or an array with the first element being the module name and the second element being an options object to be passed to that plugin.

  Example:

  ``` js
  System.config({
    vue: {
      postcss: [
        'postcss-nested',
        ['autoprefixer', { browsers: 'last 2 versions' }]
      ]
    }
  })
  ```

- An object containing `options` (passed to `postcss.process()`) and `plugins`. The parser and stringifier options can also be string module names and will be resolved automatically.

  Example:

  ``` js
  System.config({
    vue: {
      postcss: {
        options: {
          parser: 'sugarss'
        },
        plugins: [...]
      }
    }
  })
  ```

If using `vue.config.js`, you can also directly provide the imported plugins instead of string module names.

## Caveats

- SystemJS' hot reload mechanism is quite limiting and it's currently not possible to support the same level of hot-reload available in `vue-loader` and `vueify`.
