module.exports = generateUbqlForUbmSysdictionary

/**
 * @param {object} params
 * @param {SyncConnection} params.conn
 */
function generateUbqlForUbmSysdictionary ({ conn }) {
  console.log('Migration script start:', __filename)

  const entity = 'ubm_sysdictionary'

  console.info(
    `Update all "${entity}.ubql" attribute from autogenerated ubql using "UB.Repository"`
  )

  console.log(`\tLoading entries of ${entity}...`)
  const entries = conn.Repository(entity).attrs('ID', 'code', 'ubql').select()
  console.log(`\t${entries.length} entries`)

  entries.forEach(({ ID, code, ubql }) => {
    if (ubql) {
      console.log(`Skip ${entity} ID=${ID} (${code}), because there is ubql value`)
      return
    }

    const generatedUbql = JSON.stringify(
      conn.Repository(code).attrs('ID', 'code', 'name').orderBy('name').ubql()
    )

    conn.query({
      entity,
      method: 'update',
      execParams: {
        ID,
        ubql: generatedUbql
      },
      __skipOptimisticLock: true
    })
    console.info('\tSet ubql="%s" for "%s"', generatedUbql, code)
  })

  console.log('Migration script finish:', __filename)
}
